package io.github.beradq.adybubbles

inline infix fun <T, R> T.pipe(func: (T) -> R): R {
    return func(this)
}

inline infix fun <T, R> T.also(func: (T) -> R): T {
    func(this)
    return this
}

fun parseLongRange(range: String): LongRange? {
    val rangeStr = range.trim()
    val start: Long
    val end: Long

    val startInclusive: Boolean = when (rangeStr[0]) {
        '[' -> true
        '(' -> false
        else -> {
            // 如果只有一个数字，则返回该数字
            return rangeStr.toLongOrNull()?.let { it..it }
        }
    }

    val endInclusive: Boolean = when (rangeStr[rangeStr.length - 1]) {
        ']' -> true
        ')' -> false
        else -> return null
    }

    val rangeParts = rangeStr.substring(1, rangeStr.length - 1).split(',').map { it.trim() }
    if (rangeParts.size != 2) {
        return null
    }

    start = rangeParts[0].toLongOrNull() ?: return null
    end = rangeParts[1].toLongOrNull() ?: return null

    return when {
        startInclusive && endInclusive -> start..end
        startInclusive && !endInclusive -> start until end
        !startInclusive && endInclusive -> (start + 1)..end
        else -> (start + 1) until end
    }
}

fun String.toLongRange() = parseLongRange(this)

fun longRangeToString(range: LongRange): String {
    val start = range.first
    val end = range.last
    if (start == end) return "$start"
    return "[$start, $end]"
}

fun LongRange.toMString() = longRangeToString(this)